import unittest
import os
import numpy as np
from neutronbraggedge.experiment_handler.tof import TOF
from neutronbraggedge.experiment_handler.lambda_wavelength import LambdaWavelength
from neutronbraggedge.experiment_handler.experiment import Experiment


class ExperimentTest(unittest.TestCase):

    def setUp(self):
        _file_path = os.path.dirname(__file__)
        self.data_path = os.path.abspath(os.path.join(_file_path, '../../data'))

    def test_experiment_value_error_when_no_tof_provided(self):
        """Assert in experiemnt - that ValueError is raised when tof array is missing"""
        self.assertRaises(ValueError, Experiment)

    def test_experiment_value_error_when_missing_argument_for_lambda_calculation(self):
        """Assert in experiment - that ValueError is raised when detector_offset or LDS are missing for lambda calculation"""
        _tof = [1., 2., 3., 4., 5., 6., 7.]
        self.assertRaises(ValueError, Experiment, _tof)
        self.assertRaises(ValueError, Experiment, _tof, None, 1)
        self.assertRaises(ValueError, Experiment, _tof, None, None, 2)
        
    def test_experiment_value_error_when_lambda_provided_and_either_lds_or_offset_missing(self):
        """Assert in experiment - that ValueError is raised when LdS and offset are missing when lambda provided"""
        _tof = [1., 2., 3., 4., 5., 6., 7.]
        _lambda = [11., 12., 13., 14., 15., 16., 17.]
        self.assertRaises(ValueError, Experiment, _tof, _lambda)

    def test_experiment_value_error_when_lambda_and_tof_not_same_size(self):
        """Assert in experiment - that ValueError is raised when tof and lambda array do not have the same size"""
        _tof = [1., 2., 3., 4., 5., 6., 7.]
        _lambda = [11., 12., 13., 14., 15.]
        self.assertRaises(ValueError, Experiment, _tof, _lambda, 10)
        
    def test_experiment_calculate_main_coefficient(self):
        """Assert in experiment - the calculation of main coefficient is correct"""
        _tof_file = os.path.join(self.data_path, 'tof.txt')
        _tof_obj = TOF(filename = _tof_file)
        _distance_source_detector_m = 1.609
        _detector_offset_s = 4500e-6
        _exp_obj = Experiment(tof = _tof_obj.tof_array,
                              distance_source_detector_m = _distance_source_detector_m,
                              detector_offset_micros = _detector_offset_s)
        self.assertAlmostEqual(2.45869e-7, _exp_obj._h_over_MnLds, delta = 0.0001)
    
    def test_experiment_calculate_lambda(self):
        """Assert in experiment - the calculation of lambda is correct"""
        _tof_file = os.path.join(self.data_path, 'tof.txt')
        _tof_obj = TOF(filename = _tof_file)
        _distance_source_detector_m = 1.609
        _detector_offset_micros = 4500
        _exp_obj = Experiment(tof = _tof_obj.tof_array,
                              distance_source_detector_m = _distance_source_detector_m,
                              detector_offset_micros = _detector_offset_micros)
        _lambda_expected = np.array([1.10664704e-09, 1.10916474e-09, 1.11168244e-09, 1.11420014e-09,
                                     1.11671784e-09, 1.11923554e-09, 1.12175324e-09, 1.12427094e-09,
                                     1.12678864e-09, 1.12930634e-09, 1.13182403e-09, 1.13434173e-09,
                                     1.13685943e-09, 1.13937713e-09, 1.14189483e-09, 1.14441253e-09,
                                     1.14693023e-09, 1.14944793e-09, 1.15196563e-09, 1.15448333e-09])
        _lambda_returned = _exp_obj.lambda_array
        self.assertTrue(_lambda_expected[0], _lambda_returned[0])
        self.assertTrue(_lambda_expected[5], _lambda_returned[5])
        self.assertTrue(_lambda_expected[-1], _lambda_returned[-1])
        self.assertTrue(len(_lambda_expected), len(_lambda_returned))
        
    def test_create_csv_lambda_file(self):
        """Assert in Experiment - the lambda file is correctly exported"""
        _tof_file = os.path.join(self.data_path, 'tof.txt')
        _tof_obj = TOF(filename = _tof_file)
        _distance_source_detector_m = 1.609
        _detector_offset_micros = 4500
        _exp_obj = Experiment(tof = _tof_obj.tof_array,
                              distance_source_detector_m = _distance_source_detector_m,
                              detector_offset_micros = _detector_offset_micros)
        _output_filename = os.path.join(self.data_path, 'remove_me.txt')
        _exp_obj.export_lambda(filename = _output_filename)
        self.assertTrue(os.path.isfile(_output_filename))
        os.remove(_output_filename) # cleanup temp file
        
    def test_create_csv_lambda_file_without_providing_name(self):
        """Assert in Experiment - no file is created if no filename is provided"""
        _tof_file = os.path.join(self.data_path, 'tof.txt')
        _tof_obj = TOF(filename = _tof_file)
        _distance_source_detector_m = 1.609
        _detector_offset_micros = 4500
        _output_file_name = os.path.join(self.data_path, 'no_file.txt')
        _exp_obj = Experiment(tof = _tof_obj.tof_array,
                              distance_source_detector_m = _distance_source_detector_m,
                              detector_offset_micros = _detector_offset_micros)
        self.assertRaises(ValueError, _exp_obj.export_lambda)

    def test_calculate_distance_source_detector(self):
        """Assert in Experiment - the distance source detector is corectly calculated"""
        _tof_file = os.path.join(self.data_path, 'tof.txt')
        _tof_obj = TOF(filename = _tof_file)
        _detector_offset_micros = 4500
        _lambda_file = os.path.join(self.data_path, 'lambda.txt')
        _lambda_obj = LambdaWavelength(filename = _lambda_file)
        _tof_array = _tof_obj.tof_array[0:20]
        _lambda_array = _lambda_obj.lambda_array[0:20]

        _exp_handler = Experiment(tof = _tof_array,
                                  lambda_array = _lambda_array,
                                  detector_offset_micros = _detector_offset_micros)
        _distance_expected = 1.609 #m
        _distance_returned = _exp_handler.distance_source_detector
        self.assertAlmostEqual(_distance_expected, _distance_returned, delta = 1e-6)
        
    def test_calculate_detector_offset(self):
        """Assert in Experiment - the detector offset is correctly calculated"""
        _tof_file = os.path.join(self.data_path, 'tof.txt')
        _tof_obj = TOF(filename = _tof_file)
        _distance_source_detector_m = 1.609
        _lambda_file = os.path.join(self.data_path, 'lambda.txt')
        _lambda_obj = LambdaWavelength(filename = _lambda_file)
        _tof_array = _tof_obj.tof_array[0:20]
        _lambda_array = _lambda_obj.lambda_array[0:20]

        _exp_handler = Experiment(tof = _tof_array,
                                  lambda_array = _lambda_array,
                                  distance_source_detector_m = _distance_source_detector_m)
        _offset_expected_micros = 4500 #micros
        _offset_calculated = _exp_handler.detector_offset_micros
        self.assertAlmostEqual(_offset_expected_micros, _offset_calculated, 
                               delta = 1e-6)

if __name__ == '__main__':
    unittest.main()



import numpy as np
import os
from ..utilities import Utilities


class TOF(object):
    """This class handles the loading of the TOF and the automatic conversion to 's'"""
    
    tof_array = []
    counts_array = []
    
    def __init__(self, filename=None, tof_array=None, units='s'):
        """Constructor of the TOF class
        
        Arguments:
        * filename: optional input file name. If file exist, data will be automatically loaded 
        (only CSV file is supported so far)
           example: file_tof.txt
                    #first row of the file
                    1.0  34
                    2.2  31
                    3.4  5
                    4.5  10
                    5.6  22
                    ...

        * tof_array: optional tof array. This argument will be ignored if filename is not None
        * units: optional units of the input tof array (default to 'seconds')

        Raises:
        * ValueError: - input file provided as the wrong format
                      - neither input file and tof_array are provided
                      
        * IOError: - file does not exist
        
        """

        if (filename is not None):
            if os.path.isfile(filename):
                self.filename = filename
                self.load_data()
            else:
                raise IOError("File does not exist")
        else:
            if (tof_array is not None):
                if (not type(tof_array) is np.ndarray):
                    self.tof_array = np.array(tof_array)
                else:
                    self.tof_array = tof_array
            else:
                raise ValueError("Please provide a tof array")

        if units is not 's':
            self.tof_array = Utilities.convert_time_units(data = self.tof_array,
                                                    from_units = units,
                                                    to_units = 's')
        
        
    def load_data(self):
        """Load the data from the filename name provided"""
        
        # only loader implemented so far !
        _ascii_array = Utilities.load_ascii(filename = self.filename, sep='')
        _tof_column = _ascii_array[:,0]
        self.tof_array = _tof_column

        _counts_column = _ascii_array[:,1]
        self.counts_array = _counts_column
        




import unittest
import os
import numpy as np
from neutronbraggedge.braggedge import BraggEdge


class TestBraggEdge(unittest.TestCase):

    def setUp(self):
        pass


    def test_raise_error_when_nothing_passed_in(self):
        """Assert an error is raised when no material are passed in"""
        self.assertRaises(ValueError, BraggEdge)
        
    def test_raise_error_when_bad_new_material_input_format(self):
        """Assert an error is raised when the new_material array format is wrong"""
        new_material = [{'wrong_name': 'Ta', 'wrong_lattice_constant': 34545}]
        self.assertRaises(ValueError, BraggEdge, None, new_material)

    def test_retrieve_correct_metadata_for_single_local_material(self):
        """Assert the correct metadata are retrieved for a single local material"""
        new_material = [{'name': 'Ta', 'lattice': 0.5, 'crystal_structure': 'BCC'}]
        _handler = BraggEdge(new_material = new_material)
        _metadata = _handler.metadata
        self.assertEqual(0.5, _metadata['lattice']['Ta'])
        self.assertEqual('BCC', _metadata['crystal_structure']['Ta'])
        
    def test_retrieve_correct_metadata_for_multi_local_material(self):
        """Assert the correct metadata are retrieved for a couple of local material"""
        new_material = [{'name': 'Ta', 'lattice': 0.5, 'crystal_structure': 'BCC'},
                        {'name': 'Ni', 'lattice': 1.5, 'crystal_structure': 'FCC'}]
        _handler = BraggEdge(new_material = new_material)
        _metadata = _handler.metadata
        self.assertEqual(0.5, _metadata['lattice']['Ta'])
        self.assertEqual(1.5, _metadata['lattice']['Ni'])
        self.assertEqual('BCC', _metadata['crystal_structure']['Ta'])
        self.assertEqual('FCC', _metadata['crystal_structure']['Ni'])
        
    def test_calculate_d_spacing_for_single_local_material(self):
        """Assert the d_spacing calculation is correct for a single local material"""
        new_material = [{'name': 'Ta', 'lattice': 5.0, 'crystal_structure': 'BCC'}]
        _handler = BraggEdge(new_material = new_material)
        _d_spacing = _handler.d_spacing
        self.assertAlmostEqual(_d_spacing['Ta'][0], 3.5355, delta = 0.0001)
        
    def test_calculate_hkl_for_single_local_material(self):
        """Assert the hkl calculation is correct for a single local material"""
        new_material = [{'name': 'Ta', 'lattice': 5.0, 'crystal_structure': 'BCC'}]
        _handler = BraggEdge(new_material = new_material)
        _hkl = _handler.hkl
        self.assertEqual(_hkl['Ta'][0], [1, 1, 0])
        self.assertEqual(_hkl['Ta'][1], [2, 0, 0])

    def test_retrieving_correct_metadata_for_Ni(self):
        """Assert the correct metadata are returned for Ni"""
        _handler = BraggEdge(material = 'Ni')
        _metadata = _handler.metadata
        self.assertAlmostEqual(3.5238, _metadata['lattice']['Ni'], delta = 0.01)
        self.assertEqual('FCC', _metadata['crystal_structure']['Ni'])
    
    def test_retrieving_correct_number_and_first_2_values_hkl_for_Si(self):
        """Assert the correct hkl first 2values are returned for Si, and the correct number"""
        _handler = BraggEdge(material = 'Si', number_of_bragg_edges = 4)
        _hkl = _handler.hkl['Si']
        self.assertEqual([1, 1, 1], _hkl[0])
        self.assertEqual([2, 0, 0], _hkl[1])
        self.assertEqual(4, len(_hkl))

    def test_calculating_d_spacing_values_for_Ni(self):
        """Assert the first 3 d_spacing are correct for Ni"""
        _handler = BraggEdge(material = 'Ni', number_of_bragg_edges = 4)
        _d_spacing = _handler.d_spacing['Ni']
        self.assertAlmostEqual(2.0345, _d_spacing[0], delta = 0.001)
        self.assertAlmostEqual(1.7619, _d_spacing[1], delta = 0.001)
        self.assertAlmostEqual(1.2459, _d_spacing[2], delta = 0.001)
        
    def test_retrieving_first_2_values_hkl_for_Fe(self):
        """Assert the correct hkl first 2 values are returned for Fe """
        _handler = BraggEdge(material = 'Fe', number_of_bragg_edges = 4)
        _hkl = _handler.hkl['Fe']
        self.assertEqual([1, 1, 0], _hkl[0])
        self.assertEqual([2, 0, 0], _hkl[1])
        self.assertEqual([2, 1, 1], _hkl[2])
        self.assertEqual([2, 2, 0], _hkl[3])

    def test_calculating_bragg_edges_for_Fe(self):
        """Assert the first 3 bragg_edges are correct for Fe"""
        _handler = BraggEdge(material = 'Fe', number_of_bragg_edges = 4)
        _bragg_edges = _handler.bragg_edges['Fe']
        self.assertAlmostEqual(4.0537, _bragg_edges[0], delta = 0.001)
        self.assertAlmostEqual(2.8664, _bragg_edges[1], delta = 0.001)
        self.assertAlmostEqual(2.3404, _bragg_edges[2], delta = 0.001)
        
    def test_printing_report(self):
        """Assert the metadata/hkl/braggedges are correctly output"""
        _handler = BraggEdge(material = 'Ni', number_of_bragg_edges = 5)

    def test_create_export_csv_no_file_raise_error(self):
        """Assert the IOError is raised when no file name given"""
        _handler = BraggEdge(material = 'Fe', number_of_bragg_edges = 4)
        self.assertRaises(IOError, _handler.export)

    def test_create_export_csv_metadata(self):
        """Assert the correct metadata data are created for Fe when using create output file"""
        _handler = BraggEdge(material = 'Fe', number_of_bragg_edges = 4)
        _metadata = _handler._format_metadata('Fe')
        self.assertEqual("Material: Fe", _metadata[0])
        self.assertEqual("Lattice : 2.8664Angstroms", _metadata[1])
        self.assertEqual("Crystal Structure: BCC", _metadata[2])
        self.assertEqual("Using local metadata Table: True", _metadata[3])
        
    def test_create_export_csv_data(self):
        """Assert the correct data are created for Fe when using create output file"""
        _handler = BraggEdge(material = 'Fe', number_of_bragg_edges = 4)
        _data = _handler._format_data('Fe')
        self.assertEqual(1, _data[0][0])
        self.assertAlmostEqual(2.02685, _data[0][3], delta = 0.0001)
        self.assertAlmostEqual(4.05370, _data[0][4], delta = 0.0001)
        self.assertAlmostEqual(1.17020, _data[2][3], delta = 0.0001)
        self.assertAlmostEqual(2.34041, _data[2][4], delta = 0.0001)
    
    def test_create_export_csv_file_created(self):
        """Assert the correct output CSV file is created"""
        _handler = BraggEdge(material = 'Fe', number_of_bragg_edges = 4)
        _filename = 'remove_me.txt'
        _handler.export(filename = _filename, file_type = 'csv')
        self.assertTrue(os.path.isfile('remove_me_Fe.txt'))
        os.remove('remove_me_Fe.txt') #cleanup temp file
    
    def test_create_export_unsuported_file_raise_error(self):
        """Assert in BraggEdge - NotImplementedError raised when trying to create unsuported output format"""
        _handler = BraggEdge(material = 'Fe', number_of_bragg_edges = 4)
        _filename = 'remove_me_Fe.txt'
        self.assertRaises(NotImplementedError, _handler.export, _filename, 'do_not_exist_yet')

    def test_calculate_experimental_lattice_with_no_input_provided(self):
        """Assert in BraggEdge - calculate exp. lattice - ValueError raised if no experimental bragg edge array provided """
        _handler = BraggEdge(material = 'Fe', number_of_bragg_edges = 4)
        self.assertRaises(ValueError, _handler.get_experimental_lattice_parameter)
        
    def test_calculate_experimental_lattice_with_value_and_error_different_size(self):
        """Assert in BraggEdge - bragg edge value and error have different sizes"""
        _handler = BraggEdge(material = 'Fe', number_of_bragg_edges = 4)
        exp_bragg_value = np.array([1,2,3])
        exp_bragg_error = np.array([0.1, 0.2])
        self.assertRaises(ValueError, _handler.get_experimental_lattice_parameter, exp_bragg_value,
                          exp_bragg_error)
        
    def test_loading_single_material_in_list(self):
        """Assert in BraggEdge - single element Al data listed in a list correctly caluclated"""
        _handler = BraggEdge(material = ['Al'], number_of_bragg_edges = 4)
        
        


if __name__ == '__main__':
    unittest.main()



import unittest
import os
import numpy as np
from neutronbraggedge.utilities import Utilities


class UtilitiesTest(unittest.TestCase):

    def setUp(self):
        _file_path = os.path.dirname(__file__)
        self.data_path = os.path.abspath(os.path.join(_file_path, '../data'))

    def get_full_path(self, file_name):
        return os.path.join(self.data_path, file_name)

    def test_convert_time_units_raise_value_error_when_no_data_provided(self):
        """Assert in utilities - ValueError raised when no data provided"""
        self.assertRaises(ValueError, Utilities.convert_time_units)
    
    def test_convert_time_units_raise_value_error_when_units_not_supported(self):
        """Assert in utilities - ValueError raised when no units converted"""
        self.assertRaises(ValueError, Utilities.convert_time_units, 45, 'bad_units', 's')
        self.assertRaises(ValueError, Utilities.convert_time_units, 45, 's', 'bad_units')
        self.assertRaises(ValueError, Utilities.convert_time_units, 45, 'bad_units', 'bad_units')

    def test_get_time_conversion_coeff(self):
        """Assert in utilities - correct time coefficient are returned"""
        self.assertEqual(1, Utilities.get_time_conversion_coeff(from_units = 's', 
                                                                to_units = 's'))
        self.assertEqual(1.e3, Utilities.get_time_conversion_coeff(from_units = 's',
                                                                   to_units = 'ms'))
        self.assertEqual(1.e6, Utilities.get_time_conversion_coeff(from_units = 's',
                                                                   to_units = 'micros'))
        self.assertEqual(1.e9, Utilities.get_time_conversion_coeff(from_units = 's',
                                                                   to_units = 'ns'))
        
        self.assertEqual(1.e-3, Utilities.get_time_conversion_coeff(from_units = 'ms',
                                                                   to_units = 's'))
        self.assertEqual(1, Utilities.get_time_conversion_coeff(from_units = 'ms',
                                                                to_units = 'ms'))
        self.assertEqual(1e3, Utilities.get_time_conversion_coeff(from_units = 'ms',
                                                                to_units = 'micros'))
        self.assertEqual(1e6, Utilities.get_time_conversion_coeff(from_units = 'ms',
                                                                to_units = 'ns'))

        self.assertEqual(1.e-6, Utilities.get_time_conversion_coeff(from_units = 'micros',
                                                                   to_units = 's'))
        self.assertEqual(1.e-3, Utilities.get_time_conversion_coeff(from_units = 'micros',
                                                                to_units = 'ms'))
        self.assertEqual(1, Utilities.get_time_conversion_coeff(from_units = 'micros',
                                                                to_units = 'micros'))
        self.assertEqual(1e3, Utilities.get_time_conversion_coeff(from_units = 'micros',
                                                                to_units = 'ns'))

        self.assertEqual(1e-9, Utilities.get_time_conversion_coeff(from_units = 'ns',
                                                                to_units = 's'))
        self.assertEqual(1e-6, Utilities.get_time_conversion_coeff(from_units = 'ns',
                                                                to_units = 'ms'))
        self.assertEqual(1e-3, Utilities.get_time_conversion_coeff(from_units = 'ns',
                                                                to_units = 'micros'))
        self.assertEqual(1, Utilities.get_time_conversion_coeff(from_units = 'ns',
                                                                to_units = 'ns'))
        
    def test_get_time_conversion_raise_error(self):
        """Assert in utilities - ValueError is raised when wrong units given"""
        self.assertRaises(ValueError, Utilities.get_time_conversion_coeff, 's', 'bad_units')
        self.assertRaises(ValueError, Utilities.get_time_conversion_coeff, 'bad_units', 's')
        self.assertRaises(ValueError, Utilities.get_time_conversion_coeff, 'bad_units', 'bad_units')


    def test_convert_time_units_single_value(self):
        """Assert in utilities - converting single time units value"""
        self.assertEqual(5, Utilities.convert_time_units(data = 5,
                                                         from_units = 's',
                                                         to_units = 's'))
        self.assertAlmostEqual(4.5e-3, Utilities.convert_time_units(data = 4500.,
                                                         from_units = 'micros',
                                                         to_units = 's'),
                               delta = 0.000000001)
        
    def test_convert_time_units_list_array(self):
        """Assert in utilities - converting list time units value"""
        _data = [1, 2, 3, 4, 5]
        self.assertTrue(all(_data == Utilities.convert_time_units(data = _data,
                                                             from_units = 's',
                                                             to_units = 's')))
        
    def tets_convert_time_units_numpy_array(self):
        """Assert in utilities - converting numpy array time units value"""
        _data = np.array([1., 2., 3., 4., 5.])
        self.assertTrue(all(_data == Utilities.convert_time_units(data = _data,
                                                             from_units = 's',
                                                             to_units = 's')))
        
    def test_array_multiply_coeff_with_no_array(self):
        """Assert in utilities - multiply nothing by a coeff value"""
        self.assertRaises(ValueError, Utilities.array_multiply_coeff)

    def test_array_multiply_coeff(self):
        """Assert in utilities - multiply array by coeff value"""
        _data = np.array([1., 2., 3., 4., 5.])
        _coeff = 2.5
        _new_data = Utilities.array_multiply_coeff( data = _data, 
                                                    coeff = _coeff)
        _expected_data = np.array([2.5, 5., 7.5, 10, 12.5]) 
        self.assertTrue(all(_expected_data == _new_data))   

    def test_array_add_coeff_with_no_array(self):
        """Assert in utilities - add nothing to a coeff value"""
        self.assertRaises(ValueError, Utilities.array_add_coeff)

    def test_array_add_coeff(self):
        """Assert in utilities - adding coeff to array"""
        _data = np.array([1., 2., 3., 4., 5.])
        _coeff = 5.
        _new_data = Utilities.array_add_coeff( data = _data, 
                                               coeff = _coeff)
        _expected_data = np.array([6., 7., 8., 9., 10.])
        self.assertTrue(all(_expected_data == _new_data))
        
    def test_array_divide_array_not_same_size(self):
        """Assert in Utilities - Numerator array not same size as denominator array"""
        _numerator = np.array([1, 2, 3])
        _denominator = np.array([1, 2])
        self.assertRaises(ValueError, Utilities.array_divide_array, _numerator, _denominator)
    
    def test_array_divide_array_works(self):
        """Assert in Utilities - Ratio of arrays works"""
        _numerator = np.array([10, 20, 30, 40, 50])
        _denominator = np.array([1, 2, 3, 4, 5])
        _ratio_expected = np.array([10, 10, 10, 10, 10])
        _ratio_returned = Utilities.array_divide_array(numerator = _numerator,
                                                       denominator = _denominator)
        self.assertTrue(all(_ratio_expected == _ratio_returned))
        
    def test_array_minus_array_raise_error(self):
        """Assert in Utilities - array1 minus array2 raise error if not same size"""
        _array1 = np.array([1, 2, 3])
        _array2 = np.array([1, 2])
        self.assertRaises(ValueError, Utilities.array_minus_array, _array1, _array2)
        
    def test_array_minus_array_works(self):
        """Assert in Utilities - array1 minus array2 works returns correct array"""
        _array1 = np.array([2, 4, 6])
        _array2 = np.array([2, 3, 4])
        _array_returned = Utilities.array_minus_array(_array1, _array2)
        _array_expected = np.array([0, 1, 2])
        self.assertTrue(all(_array_expected == _array_returned))

    def test_load_csv_raise_value_error(self):
        """Assert in Utilities - testing load_csv - ValueError raised when wrong file format"""
        input_file = self.get_full_path('bad_file.txt')
        self.assertRaises(ValueError, Utilities.load_csv, input_file)

    def test_load_ascii_raise_value_error(self):
        """Assert in Utilities - testing load_ascii - ValueError is raised when file format is wrong"""
        input_file = self.get_full_path('bad_file.txt')
        self.assertRaises(ValueError, Utilities.load_ascii, input_file)

    def test_load_ascii_with_space_separator(self):
        """Assert in Utilities - testing load_ascii - space separated file read correctly"""
        input_file = self.get_full_path('good_tof.txt')
        _array = Utilities.load_ascii(filename = input_file,
                                      sep = ' ')[0]
        expected_start_of_array = np.array([[1., 20.],[2., 21.]])[0]
        returned_array = _array[0:2]
        self.assertTrue(all(expected_start_of_array == returned_array))

        
if __name__ == '__main__':
    unittest.main()



import os
curdir = os.path.abspath(os.path.dirname(__file__))
local_table = os.path.join(curdir, 'data/material_list.dat')
config_file = os.path.join(curdir, 'config.cfg')

# This makes Travis happy !
#local_table = 'python/neutronbraggedge/data/material_list.dat'
#config_file = 'python/config.cfg'



import numpy as np
import configparser
from ..config import config_file as config_config_file
from ..braggedges_handler.braggedge_calculator import BraggEdgeCalculator


class Lattice(object):
    """When the Bragg Edges, crystal structure and hkl are known, this class calculates the
    lattice parameter
    """
    
    space = 75
    
    material = None
    crystal_structure = None
    use_local_metadata = True
    bragg_edge_array = None
    
    def __init__(self, material=None, 
                 crystal_structure=None, 
                 bragg_edge_array=None,
                 bragg_edge_error_array=None,
                 use_local_metadata_table=True):
        
        self.material = material
        self._crystal_structure = crystal_structure  
        self.crystal_structure = crystal_structure #only used to run test
        self.use_local_metadata = use_local_metadata_table
        self.bragg_edge_array = self._format_array(bragg_edge_array)
        self.bragg_edge_error_array = self._format_array(bragg_edge_error_array)
    
        #retrieve hkl
        o_bragg_calculator = BraggEdgeCalculator(structure_name = crystal_structure, 
                                                lattice = None, 
                                                number_of_set = len(bragg_edge_array))
        o_bragg_calculator.calculate_hkl()
        self.hkl = o_bragg_calculator.hkl
        
        self._calculate()
    
    @property
    def crystal_structure(self):
        return self._crystal_structure
    
    @crystal_structure.setter
    def crystal_structure(self, structure_name):
        _config_file = config_config_file
        config_obj = configparser.ConfigParser()
        config_obj.read(_config_file)
        self._list_structure = config_obj['DEFAULT']['list_structure']
        
        if not (structure_name in self._list_structure):
            raise ValueError("Structure name should be in the list " , self._list_structure)
        self._crystal_structure = structure_name
        
    def _format_array(self, bragg_edge_array):
        """Make sure that None value are replaced by np.NaN"""
        _bragg_edge_array_formated = []

        if bragg_edge_array is None:
            sz = len(self.bragg_edge_array)
            _bragg_edge_array_formated = np.zeros((sz))
            return _bragg_edge_array_formated

        for _value in bragg_edge_array:
            if _value is None:
                _value = np.NaN
            _bragg_edge_array_formated.append(_value)
        _bragg_edge_array_formated = np.array(_bragg_edge_array_formated)
        return _bragg_edge_array_formated
        
    def _calculate(self):
        """calculate the lattice parameters step by step"""
        self._match_bragg_edge_with_hkl()
        self._calculate_lattice_array()
        self._calculate_lattice_statistics()

    def _match_bragg_edge_with_hkl(self):
        """Match each bragg edge with its equivalent hkl"""
        _bragg_edge_array = self.bragg_edge_array
        _bragg_edge_array_error = self.bragg_edge_error_array
        
        zipped = zip(self.hkl, _bragg_edge_array, _bragg_edge_array_error)
        self.hkl_bragg_edge = list(zipped)
        
    def display_hkl_bragg_edge(self):
        """Display the hkl_bragg_edge list using pretty table form"""
        print("hkl Bragg Edge Table")
        print("=" * self.space)
        print("hkl \t\t Bragg Edge Value\t Bragg Edge Error \t Lattice")
        print("-" * self.space)
        _lattice_array = self.lattice_array
        for _index, _row in enumerate(self.hkl_bragg_edge):
            _key = _row[0]
            _value = _row[1]
            _error = _row[2]
            _lattice = _lattice_array[_index]
            if np.isnan(_error):
                print("%r\t %.5f \t\t\t %.5f \t\t\t %.5f" %(_key, _value, _error, _lattice))
            else:
                print("%r\t %.5f \t\t %.5f \t\t %.5f" %(_key, _value, _error, _lattice))
        print("-" * self.space)
        print()
        return True

    def _calculate_lattice_array(self):
        """Calculate the array of lattice parameters"""
        _hkl_bragg_edge = self.hkl_bragg_edge
        _lattice_array = []
        _lattice_error_array = []
        for _row in _hkl_bragg_edge:
            _hkl = _row[0]
            _bragg_edge = _row[1]
            _bragg_error = _row[2]
            [_lattice, _lattice_error] = self._calculate_lattice_coefficient(hkl = _hkl,
                                                                             bragg_edge = _bragg_edge,
                                                                             bragg_error = _bragg_error)
            _lattice_array.append(_lattice)
            _lattice_error_array.append(_lattice_error)

        self.lattice_array = _lattice_array
        self.lattice_error = _lattice_error_array
            
    def _calculate_lattice_coefficient(self, hkl=None, bragg_edge=None, bragg_error=None):
        """Calculate the lattice coefficient for the given set of hkl and bragg edge"""
        _h, _k, _l = hkl
        _term1 = np.sqrt(_h**2 + _k**2 + _l**2)
        _term2 = bragg_edge/2.
        
        _lattice = _term2 * _term1
        _lattice_error = _term1 * bragg_error / 2.

        return [_lattice, _lattice_error]
    
    def _calculate_lattice_statistics(self):
        """Calculate the statistics of the lattice array
        - median 
        - average
        - mean
        - std (standard deviation)
        - min
        - max
        """
        _lattice_statistics = {}
        
        #min
        _min = np.nanmin(self.lattice_array)
        _lattice_statistics['min'] = _min
        
        #max
        _max = np.nanmax(self.lattice_array)
        _lattice_statistics['max'] = _max
        
        #median
        _median = np.nanmedian(self.lattice_array)
        _lattice_statistics['median'] = _median
        
        #mean
        _mean = np.nanmean(self.lattice_array)
        _error = self._calculate_mean_error(self.lattice_error)
        _lattice_statistics['mean'] = [_mean, _error]
        
        #std
        _std = np.nanstd(self.lattice_array)
        _lattice_statistics['std'] = _std
        
        self.lattice_statistics = _lattice_statistics
       
    def  _calculate_mean_error(self, lattice_error):
        _mean_error = 0
        _index = 0
        _sum = 0
        for _error in lattice_error:
            if not np.isnan(_error):
                _step1 = _error * _error
                _sum += _step1
                _index += 1
        _mean_error = np.sqrt(_sum)/ _index
        return _mean_error
        
    def display_lattice_statistics(self):
        """Display the lattice statistics using a pretty table form"""
        _lattice_statistics = self.lattice_statistics
        print("Lattice Statistics")
        print("=" * self.space)
        print("min: %.5f" %_lattice_statistics['min'])
        print("max: %.5f" %_lattice_statistics['max'])
        print("median: %.5f" %_lattice_statistics['median'])
        print("mean: %.5f +/- %.5f" %(_lattice_statistics['mean'][0], _lattice_statistics['mean'][1]))
        print("std: %.5f" %_lattice_statistics['std'])
        print("-" * self.space)
        print("")
    
    def display_recap(self):
        """Display a summary of input and outputs"""
        print(" -- Recap --")
        print("=" * self.space)
        print("Material: %r" %self.material)
        print("Crystal Structure: %r" %self._crystal_structure)
        print("-" * self.space)
        print("")
        
        self.display_hkl_bragg_edge()
        self.display_lattice_statistics()
        
        


import numpy as np
import os
from ..utilities import Utilities


class LambdaWavelength(object):
    """This class handles the loading of the Lambda"""
    
    def __init__(self, filename=None, data=None):
        """Constructor of the LambdaWavelength class
        
        Arguments:
        * filename: optional input file name if data array is provided
        If file exist, data will be automatically loaded 
        (only CSV file is supported so far)
           example: file_lambda.txt
                    #first row of the file
                    1.
                    2.
                    3.
                    4.
                    5.
        * data: optional if filename name provided. Array of lambda

        Raises:
        * ValueError: - input file provided as the wrong format
                      - neither input file and tof_array are provided
                      
        * IOError: - file does not exist
        
        """

        if (filename is not None):
            if os.path.isfile(filename):
                self.filename = filename
                self.load_data()
            else:
                raise IOError("File does not exist")
        else:
            if (data is not None):
                if (not type(data) is np.ndarray):
                    self.lambda_array = np.array(data)
                else:
                    self.lambda_array = data
            else:
                raise ValueError("Please provide a lambda array")

        
    def load_data(self):
        """Load the data from the filename name provided"""
        
        # only loaded implemented so far !
        self.lambda_array = Utilities.load_csv(filename = self.filename)
        


from .material_handler.retrieve_material_metadata import RetrieveMaterialMetadata
from .braggedges_handler.braggedge_calculator import BraggEdgeCalculator
from .utilities import Utilities
import os


class BraggEdge(object):
    """This is from where the user will retrieve all metadata and calculation
        
    Variables:
        
      From **python**, first you need to import the package

    >>> from neutronbraggedge.braggedge import BraggEdge
    
    For a particular element you can retrieve:
     - lattice parameter
     - h, k and l values
     - Crystal structure
     - bragg edges values
     
    For this example, we are retrieving the data for *Fe* and we are only
    interested by the first *4* crystal orientation.
    
    >>> _handler = BraggEdge(material = 'Fe', number_of_bragg_edges = 4)
    >>> print("Crystal Structure is: %s" %_handler.metadata['cyrstal_structure]))
    'BCC'
    >>> print("Lattice is %.2f" %_handler.metadata['lattice'])
    2.87
    >>> print("hkl are: " , _handler.hkl)
    hkl are: [][1,1,0],[2,0,0],[2,1,1],[2,2,0]]
    >>> print("bragg edges are: ", _handler.bragg_edges)
    bragg edges are: [2.0268, 1.4332, 1.1702, 1.0134]
    
    
    It is also possible to display all metadata at once
    
    >>> print(_handler)
    ===================================
    Material: Fe
    Lattice: 2.8664A
    Crystal Structure: BCC
    Using local metadata Table: True
    ===================================
     h | k | l |   d(A)  |    BraggEdge
    ===================================
     1 | 1 | 0 |  2.0269 |    4.0537
     2 | 0 | 0 |  1.4332 |    2.8664
     2 | 1 | 1 |  1.1702 |    2.3404
     2 | 2 | 0 |  1.0134 |    2.0269
    ===================================
    
    Then you can export the resulting metadata into a CSV file
    
    >>> _handler.export(filename = 'my_file_name.txt')
    
    """
    
    hkl = None
    metadata = None
    bragg_edges = None
    d_spacing = None
    
    def __init__(self, material=None, 
                 new_material = None,
                 number_of_bragg_edges=10, 
                 use_local_metadata_table=True):
        """
        Constructor
        
        Arguments:
           - material: name of the material such as 'Ni', 'Fe' ...
           - new_material: dictionary of new materials defined as
              [{'name': 'Ta',
               'lattice': 0.333,
               'crystal_structure': 'FCC'},
               {'name': 'Ur',
               'lattice': 0.5555,
               'crystal_structure': 'BCC'}]
           - number_of_bragg_edge:  Default 10. Number of row to display and calculate data for.
           - use_local_metadata_table: default True. Use local defined table to retrieve lattice parameters,
                                     crystal structure. If False, will go to wiki web page.
        
        """

        if material is None:
            if new_material is None:
                raise ValueError("No material or new_material defined!")
            else:
                #parse dictionary
                list_material = []
                try:
                    
                    for _element in new_material:
                        _name = _element['name']
                        list_material.append(_name)
                        _lattice_constant = _element['lattice']
                        _crystal_structure =  _element['crystal_structure']
                except:
                    raise ValueError("Check the format of the new element array!")
                    
                material = list_material

        if not (type(material) is list):
            material = [material]

        self.material = material
        self.number_of_bragg_edges = number_of_bragg_edges
        self.use_local_metadata_table = use_local_metadata_table
        
        self._retrieve_metadata(new_material = new_material)
        self._calculate_hkl()
        self._calculate_braggedges()
        
    def get_experimental_lattice_parameter(self, experimental_bragg_edge_values = None,
                                    experimental_bragg_edge_error = None):
        """calculates the experimental lattice parameter values given an array of 
        bragg edge values"""
        
        if experimental_bragg_edge_error is None:
            raise ValueError("Please provide an array of bragg edge values")
        
        if experimental_bragg_edge_error is not None:
            if len(experimental_bragg_edge_error) != len(experimental_bragg_edge_values):
                raise ValueError("Make sure exp. bragg edge value and error have the same size!")
        
        _calculator = self._calculator
        _calculator.calculate_lattice_array(experimental_bragg_edge_values,
                                            experimental_bragg_edge_error)
        exp_lattice_parameter = _calculator.lattice_experimentatl
        return exp_lattice_parameter
        
    def _retrieve_metadata(self, new_material=None):
        """This method retrieves the lattice and crystal structure of the material"""
        _lattice = {}
        _crystal_structure = {}

        if new_material is None: #retrieve infos from ascii table
            for _material in self.material:
                _handler = RetrieveMaterialMetadata(material = _material,
                                                    use_local_table = self.use_local_metadata_table)
                _lattice[_material] = _handler.lattice
                _crystal_structure[_material] = _handler.crystal_structure
                
        else: #local infos
            for _element in new_material:
                _material = _element['name']
                _local_lattice = _element['lattice']
                _local_crystal_structure =  _element['crystal_structure']

                _lattice[_material] = _local_lattice
                _crystal_structure[_material] = _local_crystal_structure

        self.lattice = _lattice
        self.crystal_structure = _crystal_structure
    
        self.metadata = {'lattice': self.lattice, 
                'crystal_structure': self.crystal_structure}

    def _calculate_hkl(self):
        """This method calculate the set of hkl up to the number_of_bragg_edges specified"""
        calculator = {}
        _hkl = {}
        
        for _material in self.material:
            _structure_name = self.metadata['crystal_structure'][_material]
            _lattice = self.metadata['lattice'][_material]
            
            _calculator = BraggEdgeCalculator(structure_name = _structure_name,
                                              lattice = _lattice,
                                              number_of_set = self.number_of_bragg_edges)
            
            _calculator.calculate_hkl()
            calculator[_material] = _calculator
            _hkl[_material] = _calculator.hkl

        self._calculator = calculator
        self.hkl = _hkl

    def _calculate_braggedges(self):
        """This method calculates the braggedges values (and the d_spacing in the same time)"""
        _d_spacing = {}
        _bragg_edges = {}

        
        for _material in self.material:
            _calculator = self._calculator[_material]
            
            _calculator.calculate_bragg_edges()
            _d_spacing[_material] = _calculator.d_spacing
            _bragg_edges[_material] = _calculator.bragg_edges

        self.d_spacing = _d_spacing
        self.bragg_edges = _bragg_edges
        
    def __repr__(self):
        """Display the metadata/hkl/d_spacing/bragg edge values"""
        nbr_ticks = 45
        
        for _material in self.material:

            print('=' * nbr_ticks)
            print("Material: %s" %_material)
            print(u"Lattice : %.4f\u212B" %self.metadata['lattice'][_material])
            print("Crystal Structure: %s" %self.metadata['crystal_structure'][_material])
            print("Using local metadata Table: %s" %self.use_local_metadata_table)
            print('=' * nbr_ticks)
            print(u" h | k | l |\t d (\u212B)  |\t BraggEdge")
            print('-' * nbr_ticks)

            _hkl = self.hkl[_material]
            _bragg_edges = self.bragg_edges[_material]
            _d_spacing = self.d_spacing[_material]
        
            for index in range(len(_d_spacing)):
                print(" %d | %d | %d |\t %.5f |\t %.5f" %(_hkl[index][0],
                                                          _hkl[index][1],
                                                          _hkl[index][2], 
                                                          _d_spacing[index],
                                                          _bragg_edges[index]))
        
            print('=' * nbr_ticks)


        return ""
        
    def export(self, filename=None, file_type='csv'):
        """Export the metadata into various file format
        
        Arguments:
        
           filename: output file name to create
           file_type: format of the file to create
              only 'csv' (simple comma separated format) is supported for now
            
        Exception:
           IOError: if no file name is provided
        
        """
        if filename is None:
            raise IOError

        for _material in self.material:
            
            _filename = self._format_filename(filename, _material)
            _metadata = self._format_metadata(_material)
            _data = self._format_data(_material)
            
            if file_type is 'csv':
                Utilities.save_csv(filename = _filename,
                                   data = _data,
                                   metadata = _metadata)
        
            else:
                raise NotImplementedError
        
        
    def _format_filename(self, filename, material):
        _filename, _extension = os.path.splitext(filename)
        new_filename = os.path.join(_filename + '_' + material + _extension)
        return new_filename
        
    def _format_metadata(self, _material):
        """Format the various metadata to put at the top of output file created"""
        _metadata = []
        _metadata.append("Material: %s" %_material)
        _metadata.append("Lattice : %.4fAngstroms" %self.metadata['lattice'][_material])
        _metadata.append("Crystal Structure: %s" %self.metadata['crystal_structure'][_material])
        _metadata.append("Using local metadata Table: %s" %self.use_local_metadata_table)
        _metadata.append("")
        _metadata.append("h, k, l, d(Angstroms), BraggEdge")
        return _metadata
    
    def _format_data(self, _material):
        """Format the data for the output file created"""
        _data = []
        _hkl = self.hkl[_material]
        _bragg_edges = self.bragg_edges[_material]
        _d_spacing = self.d_spacing[_material]
        for index in range(len(_d_spacing)):
            _data.append([_hkl[index][0],
                         _hkl[index][1],
                         _hkl[index][2],
                         _d_spacing[index],
                         _bragg_edges[index]])
        return _data




import unittest
import os
from neutronbraggedge import config
from neutronbraggedge.material_handler.retrieve_metadata_table import RetrieveMetadataTable


class TestRetrieveMetadataTable(unittest.TestCase):

    def setUp(self):
        _config_file = config.config_file
        self._config_file = os.path.abspath(_config_file)

    def test_retrieve_table_from_url(self):
        """checking if the table is correctly loaded from URL"""
       
        retrieve_meta = RetrieveMetadataTable(use_local_table=False)
        _table = retrieve_meta.get_table()
        _shape = _table.shape

        nbr_column = 3
        self.assertEqual(_shape[1], nbr_column)
        
        value_0_0 = 'Diamond (FCC)'
        self.assertEqual(value_0_0, _table.values[0][1])

    def test_retrieve_local_table(self):
        """checking if the local table is correctly loaded"""
        
        retrieve_meta = RetrieveMetadataTable()
        _table = retrieve_meta.get_table()
        _shape = _table.shape
        
        nbr_column = 3
        self.assertEqual(_shape[1], nbr_column)
        
        value_0_0 = 'Diamond (FCC)'
        self.assertEqual(value_0_0, _table.values[0][1])


if __name__ == '__main__':
    unittest.main()



"""
This class will retrieve the table from the URL and reformat it to be able to 
quickly retrieve the metadata for a given material
"""
#import os
import pandas as pd
import configparser
from ..config import config_file as config_config_file
from ..config import local_table as config_local_table


class RetrieveMetadataTable(object):
    """ Metadata table retriever 
    
    This class retrieves the metadata table that will allow us to get the lattice
    parameter and the crystal structure for a given material. 
    
    
    By default the program will retrieve the local version, but the web version can be retrieved
    by using the local_version=False flag. In this case, the table is retrieved from the following
    web page: `Lattice constant 
    <https://en.wikipedia.org/wiki/Lattice_constant>`.  

    >>> from braggedge.material_hanlder.retrieve_metadata_table import RetrieveMetadataTable
    >>> retrieve_local_meta = RetrieveMetadataTable()
    >>> _table = retrieve_local_meta.get_table()
    
    >>> retrieve_url_meta = RetrieveMetadataTable()
    >>> _table = retrieve_url_meta.get_table(use_local_table = False)
    
    """
    
    def __init__(self, use_local_table=True):
        self.use_local_table = use_local_table
        if not use_local_table: 
            self._retrieve_url()        
        
    def _retrieve_url(self):
        """retrieve the default url defined in the top config file"""
        self._config_file = config_config_file
        config_obj = configparser.ConfigParser()
        config_obj.read(self._config_file)
        self.url = config_obj['DEFAULT']['material_metadata_url']
        
    def retrieve_table(self):
        """retrieve the table that contain the material/lattice parameters....
        by default, the local version is retrieved first, but the web version can
        be selected instead by using False on use_local_table flag
        
        """
        if self.use_local_table:
            self.retrieve_table_local()
        else:
            self.retrieve_table_from_url()
            
    def retrieve_table_local(self):
        """retrieve the local table"""
        self._local_table_file = config_local_table
        local_table = pd.read_csv(self._local_table_file)
        _table = local_table.set_index("Material")
        self.table = _table

    def retrieve_table_from_url(self):
        """retrieve the table using the url defined in the config.cfg file"""
        table_list = pd.read_html(self.url)
        self.raw_table = table_list[0]
        self.format_table_from_url()
        
    def format_table_from_url(self):
        """reformat the table from the url to easily extrade the metadata"""
        _table = self.raw_table
        _table.columns = _table.values[0][:]
        _table = _table[1:]
        _table = _table.set_index('Material')
        self.table = _table

    def get_table(self):
        """return the table (via url or locally) according to flag used
        
        Args:
        use_local_table (boolean): get the local table or via the url defined in the config file (default True)

        Returns:
        Pandas table of material/lattice parameters ...

        """
        self.retrieve_table()
        return self.table



import sys
import os
import numpy as np
import configparser
from .structure_handler import StructureHandler
from ..config import config_file as config_config_file


class BraggEdgeCalculator(object):
    """
    This class calculates the h, k, and l values allowed for the given structure.
    The number of h,k,l set is by default set to 10 but can be changed
    
    Args:
    structure_name: default 'FCC'. Must be either ['FCC', 'BCC']
    
    """
    
    def __init__(self, structure_name="FCC", lattice=None, number_of_set=10):
        self.structure = structure_name #only used to test validity of input
        self._structure = structure_name
        self._number_of_set = number_of_set
        self.lattice = lattice

    @property
    def structure(self):
        return self._structure
    
    @structure.setter
    def structure(self, structure_name):
        
        _config_file = config_config_file
        print("config file ({}) exists? {}".format(_config_file, os.path.exists(_config_file)))
        #_config_file = os.path.abspath('../config.cfg')
        config_obj = configparser.ConfigParser()
        config_obj.read(_config_file)
        self._list_structure = config_obj['DEFAULT']['list_structure']
        
        if not (structure_name in self._list_structure):
            raise ValueError("Structure name should be in the list " , self._list_structure)
        self._structure = structure_name
        
    def calculate_hkl(self):
        _structure_handler = StructureHandler(structure = self._structure,
            number_of_set = self._number_of_set)      
        self.hkl = _structure_handler.hkl
        
    def calculate_bragg_edges(self):
        """This calculate the d_spacing and bragg edges of the various h, k and l"""
        if self.lattice is None:
            raise ValueError
        
        _bragg_edges_array = []
        _d_spacing = []
        for _hkl in self.hkl:
            _result = self._calculate_individual_bragg_edge(lattice = self.lattice,
                                                            h = _hkl[0],
                                                            k = _hkl[1],
                                                            l = _hkl[2])
            _d_spacing.append(_result)
            _bragg_edges_array.append(2. * _result)
        self.bragg_edges = _bragg_edges_array
        self.d_spacing = _d_spacing
            
    def _calculate_individual_bragg_edge(self, lattice=None,
                                         h=1, k=1, l=1):
        _den = np.sqrt(h**2 + k**2 + l**2)
        return float(lattice)/_den
        
    #def _calculate_individual_lattice(self, h=1, k=1, l=1, bragg_edge_value=0, bragg_edge_error=0):
        #_num = np.sqrt(h**2 + k**2 + l**2)
        #_value = float(bragg_edge_value/2.)*_num
        #_error = float(bragg_edge_error/2.)*_num
        #return [_value, _error]
        
    #def calculate_lattice_array(self, exp_bragg_edge_value, exp_bragg_edge_error=None):
        #"""calculate the lattice given the hkl and bragg edge value"""
        #_lattice_array = []
        #_lattice_error_array = []
        #for _index, _hkl in enumerate(exp_bragg_edge_value):
            #_hkl = self.hkl[_index]
            #_bragg = exp_bragg_edge_value[_index]
            
            #if exp_bragg_edge_error is not None:
                #_error = exp_bragg_edge_error[_index]
            #else:
                #_error = 0
            #[_result, _result_error] = self._calculate_individual_lattice(h = _hkl[0],
                                                                   #k = _hkl[1],
                                                                   #l = _hkl[2],
                                                                   #bragg_edge_value = _bragg,
                                                                   #bragg_edge_error = _error)
            #_lattice_array.append(_result)
            #_lattice_error_array.append(_result_error)
        
        #self.lattice_experimental = _lattice_array
        #self.lattice_error_experimental = _lattice_error_array
            
        #self.average_lattice_experimental = self._calculate_average_lattice_experimental()
        
    #def _calculate_average_lattice_experimental(self):
        #_lattice_exp = self.lattice_experimental
        #_lattice_error_exp = self.lattice_error_experimental
        
        #_mean_value = np.mean(_lattice_exp)
        #_sum_error = 0
        #for _error in _lattice_error_exp:
            #_sum_error += _error * _error
        
        #_mean_error = np.sqrt(_sum_error)
        
        #self.mean_lattice_experimental = [_mean_value, _mean_error]
        
        
        
        


import unittest
import os
from neutronbraggedge.braggedges_handler.structure_handler import StructureHandler
from neutronbraggedge.braggedges_handler.structure_handler import FCCHandler


class TestBraggEdgesHandler(unittest.TestCase):

    def setUp(self):
        pass

    def test_calling_with_wrong_structure(self):
        """checking that calling an unknown structure raises an error"""
        self.assertRaises(ValueError, StructureHandler, "HCC")
        
    def test_getting_the_right_first_hkl_for_BCC(self):
        """assert that the first h,k,l values are correct"""
        _handler = StructureHandler("BCC", 1)
        _list_hkl = _handler.hkl 
        self.assertEqual([1, 1, 0], _list_hkl[0])

    def test_getting_the_right_amount_of_hkl_for_BCC(self):
        """assert that the right number of hkl set is returned for BCC"""
        _handler = StructureHandler("BCC", 10)
        _list_hkl = _handler.hkl
        _nbr_hkl = len(_list_hkl)
        self.assertEqual(10, _nbr_hkl)
        
    def test_getting_the_right_first_hkl_value_for_BCC(self):
        """assert that the first few hkl set calculated are correct for BCC"""
        _handler = StructureHandler("BCC", 10)
        _list_hkl = _handler.hkl
        _nbr_hkl = len(_list_hkl)
        self.assertEqual([2, 0, 0], _list_hkl[1])
        self.assertEqual([2, 1, 1], _list_hkl[2])
        self.assertEqual([2, 2, 0], _list_hkl[3])
        self.assertEqual([2, 2, 2], _list_hkl[4])
        
    def test_getting_the_right_amount_of_hkl_for_FCC(self):
        """assert that the right number of hkl set is returned for FCC"""
        _handler = StructureHandler("FCC", 10)
        _list_hkl = _handler.hkl
        _nbr_hkl = len(_list_hkl)
        self.assertEqual(10, _nbr_hkl)
        
    def test_getting_the_right_first_hkl_value_for_FCC(self):
        """assert that the first few hkl set calculated are correct for FCC"""
        _handler = StructureHandler("FCC", 10)
        _list_hkl = _handler.hkl
        _nbr_hkl = len(_list_hkl)
        self.assertEqual([1, 1, 1], _list_hkl[0])
        self.assertEqual([2, 0, 0], _list_hkl[1])
        self.assertEqual([2, 2, 0], _list_hkl[2])
        self.assertEqual([2, 2, 2], _list_hkl[3])
        self.assertEqual([3, 1, 1], _list_hkl[4])
        
    def test_is_even_algorith(self):
        """assert that is_even algorithm is correct"""
        _fcc = FCCHandler(10)
        _is_even = _fcc._is_even(0)
        self.assertTrue(_is_even)
        _is_even = _fcc._is_even(1)
        self.assertFalse(_is_even)
        
    def test_same_parity_algorithm(self):
        """assert that same_parity algorithm is correct"""
        _fcc = FCCHandler(10)
        _same_parity = _fcc._same_parity(1, 1, 1)
        self.assertTrue(_same_parity)
        _same_parity = _fcc._same_parity(1, 1, 2)
        self.assertFalse(_same_parity)
        
        
if __name__ == '__main__':
    unittest.main()



import unittest
import os
from neutronbraggedge.material_handler.retrieve_material_metadata import RetrieveMaterialMetadata


class TestRetrieveMaterialMetadata(unittest.TestCase):

    def setUp(self):
        pass

    def test_retrieve_lattice_of_si_via_web(self):
        """Assert in RetrieveMaterialMetadata - value of lattice retrieved from Si via web table"""
        retrieve_material = RetrieveMaterialMetadata(material = 'Si',
                                                     use_local_table = False)
        lattice_expected = 5.431
        self.assertAlmostEqual(lattice_expected, retrieve_material.lattice, delta=0.001)

    def test_retrieve_lattice_of_si_via_local_table(self):
        """Assert in RetrieveMaterialMetadata - value of lattice retrieved from Si via local table"""
        retrieve_material = RetrieveMaterialMetadata(material = 'Si',
                                                     use_local_table = True)
        lattice_expected = 5.431591 #higher precision on local table
        self.assertEqual(lattice_expected, retrieve_material.lattice)

    def test_raise_NameError_no_arguments(self):
        """Assert in RetrieveMaterialMetadata - NameError is raised when no arguments is given"""
        self.assertRaises(NameError, RetrieveMaterialMetadata)
        
    def test_raise_NameError_material_unknown(self):
        """Assert in RetrieveMaterialMetadata - NameError is raised if material is unknown"""
        self.assertRaises(KeyError, RetrieveMaterialMetadata, 'unknown')

    def test_retrieving_full_list_of_material(self):
        """Assert in RetrieveMaterialMetadata - Full list of material available returned"""
        retrieve_material = RetrieveMaterialMetadata(material = 'all',
                                                     use_local_table = False)
        
        list_returned = retrieve_material.full_list_material()
        list_expected_3_first = ['C (diamond)', 'Si', 'Ge']
        self.assertTrue(all(list_returned[0:3] == list_expected_3_first))
        

if __name__ == '__main__':
    unittest.main()



import unittest
import os
import numpy as np
from neutronbraggedge.braggedges_handler.braggedge_calculator import BraggEdgeCalculator


class TestBraggEdgesHandler(unittest.TestCase):

    def setUp(self):
        pass

    def test_calling_without_arguments(self):
        """Testing that class can be called with default arguments from python2 and 3"""
        _handler = BraggEdgeCalculator()
        self.assertEqual("FCC", _handler.structure)
        
    def test_braggedge_calculator_eror_when_bad_structure_given(self):
        """Assert in braggedge calculator - error is raised when bad structure given"""
        _structure_name = "FakeStructure"
        self.assertRaises(ValueError, BraggEdgeCalculator, _structure_name)

    def test_right_structure_name_is_passed_in_constructor(self):
        """Assert that structure name passed in constructor is correctly used"""
        _structure_name = "BCC"
        _handler = BraggEdgeCalculator(structure_name = _structure_name)
        self.assertEqual("BCC", _handler.structure)
        
    def test_right_structure_name_is_passed_in_assigned(self):
        """Assert that structure name assigned is correctly saved"""
        _structure_name = "BCC"
        _handler = BraggEdgeCalculator()
        _handler.structure = _structure_name
        self.assertEqual("BCC", _handler.structure)
        
    def test_right_hkl_number_calculated_for_BCC(self):
        """Assert that the right number of hkl sets is returned for BCC"""
        _structure_name = "BCC"
        _handler = BraggEdgeCalculator(structure_name = _structure_name, 
                                             number_of_set = 5)
        _handler.calculate_hkl()
        _hkl = _handler.hkl
        self.assertEqual(5, len(_hkl))
        
    def test_right_hkl_number_calculated_for_FCC(self):
        """Assert that the right number of hkl sets is returned for FCC"""
        _structure_name = "FCC"
        _handler = BraggEdgeCalculator(structure_name = _structure_name, 
                                             number_of_set = 5)
        _handler.calculate_hkl()
        _hkl = _handler.hkl
        self.assertEqual(5, len(_hkl))

    def test_right_hkl_set_is_calculated_for_FCC(self):
        """Assert that the right set of hkl sets is returned for FCC"""
        _structure_name = "FCC"
        _handler = BraggEdgeCalculator(structure_name = _structure_name, 
                                             number_of_set = 5)
        _handler.calculate_hkl()
        _hkl = _handler.hkl
        self.assertEqual([1, 1, 1], _hkl[0])
        self.assertEqual([2, 0, 0], _hkl[1])
        self.assertEqual([2, 2, 0], _hkl[2])
        self.assertEqual([2, 2, 2], _hkl[3])
        self.assertEqual([3, 1, 1], _hkl[4])    
           
    def test_right_hkl_number_calculated_for_BCC(self):
        """Assert that the right number of hkl sets is returned for BCC"""
        _structure_name = "BCC"
        _handler = BraggEdgeCalculator(structure_name = _structure_name, 
                                             number_of_set = 5)
        _handler.calculate_hkl()
        _hkl = _handler.hkl
        self.assertEqual(5, len(_hkl))

    def test_right_hkl_set_is_calculated_for_BCC(self):
        """Assert that the right set of hkl sets is returned for BCC"""
        _structure_name = "BCC"
        _handler = BraggEdgeCalculator(structure_name = _structure_name, 
                                             number_of_set = 5)
        _handler.calculate_hkl()
        _hkl = _handler.hkl
        self.assertEqual([1, 1, 0], _hkl[0])
        self.assertEqual([2, 0, 0], _hkl[1])
        self.assertEqual([2, 1, 1], _hkl[2])
        self.assertEqual([2, 2, 0], _hkl[3])
        self.assertEqual([2, 2, 2], _hkl[4])

    def test_calculate_bragg_edges_algorithm_fail_when_no_lattice_given(self):
        """Assert that ValueError is correctly raised when no lattice is provided"""
        _handler = BraggEdgeCalculator(structure_name = "BCC")
        self.assertRaises(ValueError, _handler.calculate_bragg_edges)

    def test_d_spacing_for_first_hkl_of_bcc(self):
        """Assert the d_spacing values for the first BCC structure are correct"""
        _handler = BraggEdgeCalculator(structure_name = "BCC", lattice=1.)
        _handler.calculate_hkl()
        _handler.calculate_bragg_edges()
        self.assertAlmostEqual(0.7071, _handler.d_spacing[0], delta=0.0001)
        self.assertAlmostEqual(0.5, _handler.d_spacing[1], delta=0.0001)
        self.assertAlmostEqual(0.4083, _handler.d_spacing[2], delta=0.0001)

    def test_bragg_edge_for_first_hkl_of_bcc(self):
        """Assert the bragg edge values for the first BCC structure are correct"""
        _handler = BraggEdgeCalculator(structure_name = "BCC", lattice=1.)
        _handler.calculate_hkl()
        _handler.calculate_bragg_edges()
        self.assertAlmostEqual(1.4142, _handler.bragg_edges[0], delta=0.0001)
        self.assertAlmostEqual(1.0, _handler.bragg_edges[1], delta=0.0001)
        self.assertAlmostEqual(0.8165, _handler.bragg_edges[2], delta=0.0001)

    def test_d_spacing_for_first_hkl_of_fcc(self):
        """Assert the d_spacing values for the first FCC structure are correct"""
        _handler = BraggEdgeCalculator(structure_name = "FCC", lattice=1.)
        _handler.calculate_hkl()
        _handler.calculate_bragg_edges()
        self.assertAlmostEqual(1.1547/2., _handler.d_spacing[0], delta=0.0001)
        self.assertAlmostEqual(1.0/2., _handler.d_spacing[1], delta=0.0001)
        self.assertAlmostEqual(0.7071/2., _handler.d_spacing[2], delta=0.0001)

    #def test_bragg_edge_for_first_hkl_of_fcc(self):
        #"""Assert the bragg edge values for the first FCC structure are correct"""
        #_handler = BraggEdgeCalculator(structure_name = "FCC", lattice=1.)
        #_handler.calculate_hkl()
        #_handler.calculate_bragg_edges()
        #self.assertAlmostEqual(1.1547, _handler.bragg_edges[0], delta=0.0001)
        #self.assertAlmostEqual(1.0, _handler.bragg_edges[1], delta=0.0001)
        #self.assertAlmostEqual(0.7071, _handler.bragg_edges[2], delta=0.0001)

    #def test_bragg_edge_calculate_exp_lattice(self):
        #"""Assert braggedge_calculator - experimental lattice correctly calculated"""
        #_handler = BraggEdgeCalculator(structure_name = "FCC", lattice=1.)
        #_handler.calculate_hkl()
        #_bragg_edge_exp_value = np.array([4.15297, 3.59671, 2.54284, 2.16852])
        #_bragg_edge_exp_error = np.array([0.00017, 0.00010, 0.00008, 0.00007])
        #_handler.calculate_lattice_array(_bragg_edge_exp_value, _bragg_edge_exp_error)
        #_lattice_exp = _handler.lattice_experimental
        #_expected_value = np.array([3.59658, 3.59671, 3.59612, 3.755987])
        #self.assertAlmostEqual(_expected_value[0], _lattice_exp[0], delta=0.00001)
        #self.assertAlmostEqual(_expected_value[1], _lattice_exp[1], delta=0.00001)
        #self.assertAlmostEqual(_expected_value[2], _lattice_exp[2], delta=0.00001)
        #self.assertAlmostEqual(_expected_value[3], _lattice_exp[3], delta=0.00001)
        #_expected_error = np.array([0.00015, 0.0001, 0.000113, 0.00012])
        #_lattice_error = _handler.lattice_error_experimental
        #self.assertAlmostEqual(_expected_error[0], _lattice_error[0], delta=0.00001)
        #self.assertAlmostEqual(_expected_error[1], _lattice_error[1], delta=0.00001)
        #self.assertAlmostEqual(_expected_error[2], _lattice_error[2], delta=0.00001)
        #self.assertAlmostEqual(_expected_error[3], _lattice_error[3], delta=0.00001)

if __name__ == '__main__':
    unittest.main()



"""
This class will automatically retrieve the lattice parameter and the crystal structure of a given
element
"""
from .retrieve_metadata_table import RetrieveMetadataTable


class RetrieveMaterialMetadata(object):
    """ Retrieve the metadata for a given material 
    
    This class retrieves the lattice parameter and the crystal structure 
    of a given material.
    
    >>> from braggedge.material_handler.retrieve_material_metadata import RetrieveMaterialMetadata
    >>> retrieve_metadata = RetrieveMaterialMetadata("Si")
    >>> print ("Structure is: %s" %retrieve_metadata.crystal_structure)
    Structure is: FCC
    >>> print ("Lattice parameter is %.2f Angstroms" %retrieve_metadata.lattice)
    Lattice parameter is 5.43 Angstroms
    
    """
    
    lattice = None
    crystal_structure = None
    
    def __init__(self, material=None, use_local_table=True):
        """Constructor that will automatically retrieve the metadata
        
        Args:
        material: mandatory string (ex: "Si")
        use_local_table: optional boolean. By default, a local table is used, but
           user can turn off the flag to retrieve data from web site 
           (`<https://en.wikipedia.org/wiki/Lattice_constant>`)
        
        Exception:
        NameError: if no material is given
        """
        if material is None:
            raise NameError("Please provide a material")
        

        self._material = material
        self.use_local_table = use_local_table

        self._retrieve_table()
        if not (material.lower() == 'all'):
            self._retrieve_metadata()
        
    def _retrieve_table(self):
        """retrieve the table """
        metadata_table = RetrieveMetadataTable()
        self.table = metadata_table.get_table()

    def full_list_material(self):
        _list_material = self.table.index.values
        return _list_material
        
    def _retrieve_metadata(self):
        """retrieve the metadata ('lattice constant','crystal structure')"""
        try:
            _metadata = self.table.loc[self._material]
        except KeyError:
            raise KeyError("Material unknown")
        self._retrieve_lattice(_metadata)
        self._retrieve_crystal_structure(_metadata)
        
    def _retrieve_lattice(self, _metadata):
        self.lattice = float(_metadata[0])
        
    def _retrieve_crystal_structure(self, _metadata):
        _full_crystal_str = _metadata[1]

        if 'FCC' in _full_crystal_str:
            _crystal_str = 'FCC'
        elif 'BCC' in _full_crystal_str:
            _crystal_str = 'BCC'
        else:
            raise NameError("Crystal Structure not supported yet!")
        self.crystal_structure = _crystal_str
    



import unittest
import numpy as np
import os
import math
from neutronbraggedge.lattice_handler.lattice import Lattice


class TestLattice(unittest.TestCase):

    def setUp(self):
        pass

    def test_lattice_error_when_bad_structure_given(self):
        """Assert in Lattice - Value Error is raised when bad structure given"""
        _crystal_structure = 'FakeStructure'
        self.assertRaises(ValueError, Lattice, None, _crystal_structure)

    def test_correctly_hkl_returned_for_Si_3entries(self):
        """Assert in Lattice - hkl array correctly calculated and reported for 3 entries"""
        _material = 'Si'
        _crystal_structure = 'FCC'
        _bragg_edge_array = np.array([1, 2, 3])
        o_lattice = Lattice(material = _material,
                            crystal_structure = _crystal_structure,
                            bragg_edge_array = _bragg_edge_array)
        _hkl_expected = [[1, 1, 1], [2, 0, 0], [2, 2, 0]]
        self.assertTrue(o_lattice.hkl == _hkl_expected)
    
    #def test_bragg_edge_correctly_formated_with_none_value(self):
        #"""Assert in Lattice - bragg edge array correctly formated with None values"""
        #_material = 'Si'
        #_crystal_structure = 'FCC'
        #_bragg_edge_array = np.array([1, None, 3])
        #o_lattice = Lattice(material = _material,
                            #crystal_structure = _crystal_structure,
                            #bragg_edge_array = _bragg_edge_array)
        #_bragg_edge_array_expected = [1, np.NAN, 3]
        #self.assertEqual(_bragg_edge_array_expected, o_lattice.bragg_edge_array)


    def test_correctly_hkl_returned_for_Si_4entries_with_None(self):
        """Assert in Lattice - hkl array correctly calculated and reported for 4 entries with None"""
        _material = 'Si'
        _crystal_structure = 'FCC'
        _bragg_edge_array = np.array([1, 2, None, 3])
        o_lattice = Lattice(material = _material,
                            crystal_structure = _crystal_structure,
                            bragg_edge_array = _bragg_edge_array)
        _hkl_expected = [[1, 1, 1], [2, 0, 0], [2, 2, 0], [2, 2, 2]]
        self.assertTrue(o_lattice.hkl == _hkl_expected)
        
    def test_bragg_edge_crystal_structure_correctly_created(self):
        """Assert in Lattice - bragg edge crystal structure correctly created"""
        _material = "Si"
        _crystal_structure = "FCC"
        _bragg_edge_array = np.array([1.1, 2.2, 3.3, 4.4])
        o_lattice = Lattice(material = _material,
                            crystal_structure = _crystal_structure,
                            bragg_edge_array = _bragg_edge_array)
        o_lattice._match_bragg_edge_with_hkl()

        _expected_first_key = [1, 1, 1]
        _expected_first_value = 1.1
        self.assertEqual(_expected_first_key, o_lattice.hkl_bragg_edge[0][0])
        self.assertEqual(_expected_first_value, o_lattice.hkl_bragg_edge[0][1])
        
        _expected_second_key = [2, 0, 0]
        _expected_second_value = 2.2
        self.assertEqual(_expected_second_key, o_lattice.hkl_bragg_edge[1][0])
        self.assertEqual(_expected_second_value, o_lattice.hkl_bragg_edge[1][1])

    def test_bragg_edge_crystal_structure_correctly_created_for_incomplete_list(self):
        """Assert in Lattice - bragg edge crystal structure correctly created for incomplete list"""
        _material = "Si"
        _crystal_structure = "FCC"
        _bragg_edge_array = np.array([1.1, None, None, 4.4])
        o_lattice = Lattice(material = _material,
                            crystal_structure = _crystal_structure,
                            bragg_edge_array = _bragg_edge_array)
        o_lattice._match_bragg_edge_with_hkl()

        _expected_first_key = [1, 1, 1]
        _expected_first_value = 1.1
        self.assertEqual(_expected_first_key, o_lattice.hkl_bragg_edge[0][0])
        self.assertEqual(_expected_first_value, o_lattice.hkl_bragg_edge[0][1])
        
        _expected_second_key = [2, 0, 0]
        _expected_second_value = np.NaN
        self.assertEqual(_expected_second_key, o_lattice.hkl_bragg_edge[1][0])
        self.assertTrue(math.isnan(o_lattice.hkl_bragg_edge[1][1]))

    def test_hkl_bragg_edge_correctly_displayed(self):
        """Assert in Lattice - hkl bragg edge correctly displayed"""
        _material = "Si"
        _crystal_structure = "FCC"
        _bragg_edge_array = np.array([1.1, 2.2, 3.3, 4.4])
        o_lattice = Lattice(material = _material,
                            crystal_structure = _crystal_structure,
                            bragg_edge_array = _bragg_edge_array)
        o_lattice._match_bragg_edge_with_hkl()
        self.assertTrue(o_lattice.display_hkl_bragg_edge())
        
    def test_lattice_coefficient_correctly_calculated(self):
        """Assert in Lattice - lattice coefficient correctly calculated for [1,1,1]"""
        _material = "Si"
        _crystal_structure = "FCC"
        _bragg_edge_array = np.array([1.1, 2.2, 3.3, 4.4])
        o_lattice = Lattice(material = _material,
                            crystal_structure = _crystal_structure,
                            bragg_edge_array = _bragg_edge_array)
        o_lattice._match_bragg_edge_with_hkl()
        o_lattice._calculate_lattice_array()
        self.assertEqual(len(_bragg_edge_array), len(o_lattice.lattice_array))
        self.assertAlmostEqual(0.952628, o_lattice.lattice_array[0], delta=0.00001)

    def test_lattice_statistics(self):
        """Assert in Lattice - lattice statistics correctly calculated for Si"""
        _material = "Si"
        _crystal_structure = "FCC"
        _bragg_edge_array = np.array([1.1, 2.2, 3.3, 4.4])
        o_lattice = Lattice(material = _material,
                            crystal_structure = _crystal_structure,
                            bragg_edge_array = _bragg_edge_array)
        o_lattice._match_bragg_edge_with_hkl()
        o_lattice._calculate_lattice_array()
        o_lattice._calculate_lattice_statistics()
        _statistics = o_lattice.lattice_statistics
        
        _delta = 1e-4
        self.assertAlmostEqual(2.549745, _statistics['std'], delta = _delta)
        self.assertAlmostEqual(0.952628, _statistics['min'], delta = _delta)
        self.assertAlmostEqual(7.621024, _statistics['max'], delta = _delta)
        self.assertAlmostEqual(3.433452, _statistics['median'], delta = _delta)
        self.assertAlmostEqual(3.860139, _statistics['mean'][0], delta = _delta)

    def test_display_statistics(self):
        """Assert in Lattice - lattice statistics correctly displayed for Si"""
        _material = "Si"
        _crystal_structure = "FCC"
        _bragg_edge_array = np.array([1.1, 2.2, 3.3, 4.4])
        o_lattice = Lattice(material = _material,
                            crystal_structure = _crystal_structure,
                            bragg_edge_array = _bragg_edge_array)
        o_lattice._match_bragg_edge_with_hkl()
        o_lattice._calculate_lattice_array()
        o_lattice._calculate_lattice_statistics()
        o_lattice.display_lattice_statistics()

    def test_display_recap(self):
        """Assert in Lattice - lattice recap displayed correctly"""
        _material = "Si"
        _crystal_structure = "FCC"
        _bragg_edge_array = np.array([1.1, 2.2, 3.3, 4.4])
        o_lattice = Lattice(material = _material,
                            crystal_structure = _crystal_structure,
                            bragg_edge_array = _bragg_edge_array)
        o_lattice._match_bragg_edge_with_hkl()
        o_lattice._calculate_lattice_array()
        o_lattice._calculate_lattice_statistics()
        o_lattice.display_recap()
        
if __name__ == '__main__':
    unittest.main()



import numpy as np
from .tof import TOF
from ..constants import mn, h
from ..utilities import Utilities

class Experiment(object):
    """Class that allows:
       - convert the TOF scale into Lambda
       - distance source - detector
       - detector time offset
       
       Arguments:
       * tof: tof array in s
       * lambda_array: mandatory only if detector_offset or distance_source_detector are unknown
       * distance_source_detector: mandatory only if lambda is the unknown parameter (m)
       * detector_offset: mandatory only if lambda is the unknown parameter (micros)
       """
    
    def __init__(self, tof=None, lambda_array=None, 
                 distance_source_detector_m=None, 
                 detector_offset_micros=None):
        if tof is None:
            raise ValueError("Missing TOF array")
        self.tof_array = tof
        self.distance_source_detector = distance_source_detector_m
        self.detector_offset_micros = detector_offset_micros
        self.lambda_array = lambda_array

        # if lambda_array is unknown, both distance_source_detector and detector_offset must be provided
        if lambda_array is None:
            if (distance_source_detector_m is None) or (detector_offset_micros is None):
                raise ValueError("Mssing arguments to calculate lambda_array")

        # if lambda_array is provided, either distance_source_detector_m or detector_offset_micros can be missing,
        # but not both
        if lambda_array is not None:
            if (distance_source_detector_m is None) and (detector_offset_micros is None):
                raise ValueError("Missing either distance_source detector or detector_offset")

            if len(lambda_array) != len(tof):
                raise ValueError("TOF and Lambda do not have the same size !")

            if distance_source_detector_m is None:
                self.calculate_distance_source_detector()
            else:
                self.calculate_detector_offset()

        else:
            self.calculate_lambda()
        
    def calculate_tof_with_detector_offset(self):
        """return the tof with detector_offset applied to it"""
        detector_offset_micros = self.detector_offset_micros
        detector_offset_s = Utilities.convert_time_units(detector_offset_micros,
                                                         from_units = 'micros',
                                                         to_units = 's')
        # apply detector offset to tof array
        _tof = self.tof_array 
        _tof_with_offset = Utilities.array_add_coeff(data = _tof,
                                                     coeff = detector_offset_s)
        return _tof_with_offset        


    def calculate_distance_source_detector(self):
        """return the distance source detector
        
        If lambda_array and tof_array are provided, the distance is calculated
        Otherwise, the distance_source_detector must be provided
        """
        _tof_with_offset = self.calculate_tof_with_detector_offset() 
        
        # calculate the constant factor
        _coeff = h / mn

        # multiply constant facor by tof array
        _numerator = Utilities.array_multiply_coeff(data = _tof_with_offset, 
                                                coeff = _coeff)

        _denominator = self.lambda_array
        
        # divide numerator by denominator
        _ratio = Utilities.array_divide_array(numerator = _numerator,
                                              denominator = _denominator)

        self.distance_source_detector = np.mean(_ratio)

    def calculate_detector_offset(self):
        """return the detector time offset value
        
        If lambda_array and tof_array are provided, the offset is calculated
        Otherwise, the detector_offset argument must be provided
        """

        # calculate the constant factor
        lSD = self.distance_source_detector
        _coeff = h / (mn * lSD)
        _MnLds_over_h = 1./_coeff
        
        # lambda / coeff
        _lambda_array = self.lambda_array
        _lambda_over_coeff = Utilities.array_multiply_coeff(data = _lambda_array,
                                                            coeff = _MnLds_over_h)

        # (lambda/coeff) - tof
        _tof = self.tof_array
        
        _detector_offset_array = Utilities.array_minus_array(array1 = _lambda_over_coeff,
                                              array2 = _tof)
        _detector_offset_s = np.mean(_detector_offset_array)
        self.detector_offset_micros = Utilities.convert_time_units(data = _detector_offset_s,
                                                                   from_units = 's',
                                                                   to_units = 'micros')
        
    
    def calculate_lambda(self):
        """return the lambda array when tof_array, distance_source_detector and 
        detector_offset are provided
        """
        _tof_with_offset = self.calculate_tof_with_detector_offset() 

        # calculate the constant factor
        lSD = self.distance_source_detector
        _coeff = h / (mn * lSD)
        self._h_over_MnLds = _coeff

        # multiply constant factor by tof array
        _lambda = Utilities.array_multiply_coeff(data = _tof_with_offset, 
                                                coeff = _coeff)
            
        self.lambda_array = _lambda
        
    def export_lambda(self, filename=None):
        """export the lambda array into a CSV data file
        
        Parameters:
        * filename: name of output file to create
        """
        if filename is None:
            raise ValueError("Please provide a file name!")
        
        _metadata = []
        _metadata.append("Lambda (Angstroms)")
        _metadata.append("")
        _metadata.append("Distance source-detector (m): %.4f" %self.distance_source_detector)
        _metadata.append("Detector offset (micros): %.4f" %self.detector_offset_micros)
        _metadata.append("")
        _metadata.append("Lambda (Angstroms)")
        
        Utilities.save_csv(filename = filename, 
                           data = self.lambda_array,
                           metadata = _metadata)
        



import unittest
import os
import numpy as np
from neutronbraggedge.experiment_handler.lambda_wavelength import LambdaWavelength


class TofTest(unittest.TestCase):

    def setUp(self):
        _file_path = os.path.dirname(__file__)
        self.data_path = os.path.abspath(os.path.join(_file_path, '../../data'))

    def test_loading_manual_lambda_array(self):
        """Assert in LambdaWavelength - manual loading of array"""
        _lambda_array = [1., 2., 3., 4., 5., 6., 7., 8., 9.]
        _lambda_handler = LambdaWavelength(data = _lambda_array)
        self.assertTrue(all(_lambda_array == _lambda_handler.lambda_array))

    def test_loading_normal_array_data(self):
        """Assert in LambdaWavelength - loading of np.array array"""
        _lambda_array = np.array([1, 2., 3., 4.])
        _lambda_handler = LambdaWavelength(data = _lambda_array)
        self.assertTrue(all(_lambda_array == _lambda_handler.lambda_array))

    def test_not_lambda_array_provided(self):
        """Assert in LambdaWavelength - no lambda array provided"""
        self.assertRaises(ValueError, LambdaWavelength)

    def test_loading_auto_lambda_array(self):
        """Assert in LambdaWavelength - auto loading of array"""
        _lambda_filename = os.path.join(self.data_path, 'lambda.txt')
        _lambda_handler = LambdaWavelength(filename = _lambda_filename)
        _lambda_expected = np.array([1.10664703784e-09, 1.10916473754e-09, 1.11168243725e-09,
                                     1.11420013696e-09, 1.11671783666e-09, 1.11923553637e-09,
                                     1.12175323607e-09, 1.12427093578e-09, 1.12678863549e-09,
                                     1.12930633519e-09, 1.1318240349e-09, 1.1343417346e-09,
                                     1.13685943431e-09, 1.13937713401e-09, 1.14189483372e-09,
                                     1.14441253343e-09, 1.14693023313e-09, 1.14944793284e-09,
                                     1.15196563254e-09, 1.15448333225e-09]) 
        self.assertTrue(all(_lambda_expected[0:5] == _lambda_handler.lambda_array[0:5]))

    def test_load_bad_file_name(self):
        """Assert in LambdaWavelength - file name is provided but does not exist"""
        _lambda_filename = os.path.join(self.data_path, 'i_do_not_exist.txt')
        self.assertRaises(IOError, LambdaWavelength, _lambda_filename)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
from setuptools import setup, find_packages
import neutronbraggedge

VERSION = neutronbraggedge.__version__

setup(
    name = "neutronbraggedge",
    version = VERSION,
    author = "Jean Bilheux",
    author_email = "bilheuxjm@ornl.gov", 
    packages = find_packages(exclude=['tests', 'notebooks']),
    package_data = { 'neutronbraggedge' : ['data/material_list.dat', 
                                           'data/full_material_list.dat']},
    include_package_data = True,
    data_files = [('neutronbraggedge', ['neutronbraggedge/config.cfg']),
                  #('neutronbraggedge/data', ['neutronbraggedge/data/material_list.dat'])
                  ],
    test_suite = 'tests',
    install_requires = [
        'numpy',
        'configparser',
        'pandas',
        'lxml',
        'html5lib',
        'beautifulsoup4',
    ],
    dependency_links = [
    ],
    description = "Bragg Edge work at the SNS",
    license = 'BSD',
    keywords = "neutron imaging bragg edge",
    url = "https://github.com/ornlneutronimaging/BraggEdge",
    classifiers = ['Development Status :: 3 - Alpha',
                   'Topic :: Scientific/Engineering :: Physics',
                   'Intended Audience :: Developers',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6'],
    # download_url = '',
)


# End of file



"""This file will handle work to retrieve the right h,k and l set of data for the various
structures
"""
#from .braggedge_values_calculator import BraggEdgeValuesCalculator
MAX_INDEX = 30

class BCCHandler(object):
    """BCC type handler
    
    For this type, h+k+l must be an even number
    
    """
    
    def __init__(self, number_of_set):
        self.hkl = []
        self.number_of_set = number_of_set
        self.calculate_hkl()

    def _hkl_generator(self, number_of_h):
        """generator that is used to produced the right hkl for cyrstal structure"""
        h, k, l = 1, 0, 0
        for h in range(1, number_of_h):
            for k in range(number_of_h):
                if k > h:
                    continue
                for l in range(number_of_h):
                    if l > k:
                        continue
                    _sum = h + k + l
                    if _sum % 2 == 0:
                        yield [h, k, l]
        
    def calculate_hkl(self):
        """calculate the list of hkl for BCC crystal structure"""
        _hkl_list = self._hkl_generator(MAX_INDEX)
        _result = []
        for i in range(self.number_of_set):
            _result.append(next(_hkl_list))
        self.hkl = _result


class FCCHandler(object):
    """FCC type handler
    
    For this type, h, k and l must have the same parity
    
    """
    
    hkl = []
    
    def __init__(self, number_of_set):
        self.hkl = []
        self.number_of_set = number_of_set
        self.calculate_hkl()
        
    def _hkl_generator(self, number_of_h):
        """generator used to produce right set of hkl parameters"""
        for h in range(1, number_of_h):
            for k in range(0, number_of_h):
                if k > h:
                    continue
                for l in range(0, number_of_h):
                    if l > k:
                        continue
                    if self._same_parity(h, k, l):
                        yield [h, k, l]    
    
    def _same_parity(self, h, k, l):
        """This function check if the 3 variables h, k, l have the same parity or not
        
        Args:
        h
        k
        l
        
        Return:
        True if h, k and l have the same parity
        False if h, k and l do not have the same parity
        
        """
        if (self._is_even(h) and 
            self._is_even(k) and
            self._is_even(l)) or (not self._is_even(h) and
                                  not self._is_even(k) and
                                  not self._is_even(l)):
            return True
        return False
    
    def _is_even(self, n):
        """Check if a variable n is even
        
        Args:
        n variable to check
        
        Return:
        True if n is even
        False is n is odd
        """
        if n % 2 == 0:
            return True
        return False

                           
    def calculate_hkl(self):
        """calculate the hkl allowed for a FCC crystal structure"""
        _hkl_list = self._hkl_generator(MAX_INDEX)
        _result = []
        for i in range(self.number_of_set):
            _result.append(next(_hkl_list))
        self.hkl = _result
        
        
class StructureHandler(object):
    """Various structure handler"""
    
    hkl = []
    
    def __init__(self, structure, number_of_set = 10):
        if not (structure in ["BCC", "FCC"]):
            raise ValueError("structure not implemented yet")

        self.structure = structure
        self.number_of_set = number_of_set
        
        if structure == 'FCC':
            _handler = FCCHandler(number_of_set = self.number_of_set)
        elif structure == 'BCC':
            _handler = BCCHandler(number_of_set = self.number_of_set)

        self.hkl = _handler.hkl
        
        


import numpy as np


class Utilities(object):
    """Utilities class"""

    list_of_time_units = ['s', 'ms', 'micros', 'ns']

    @staticmethod
    def convert_time_units(data=None, from_units='micros', to_units='s'):
        """convert the time units
        
        Parameters:
        * vdata: single data or array of value to convert
        * from_units: default 'micros'. Must be either ['s','micros','ns']
        * to_units: default 's'. Must be either ['s', 'micros', 'ns']
        
        """
        if data is None:
            raise ValueError("Please provide data to convert")

        list_of_time_units = Utilities.list_of_time_units
        if ( not (from_units in list_of_time_units) or
             not (to_units in list_of_time_units)):
            raise ValueError("Units convertion not supported")
    
        coeff = Utilities.get_time_conversion_coeff(from_units = from_units,
                                               to_units = to_units)
        
        if (type(data) is list):
            data = np.array(data)
        
        return data * coeff
    
    @staticmethod
    def get_time_conversion_coeff(from_units='micros', to_units='s'):
        """return the coefficient to use to convert from first units to second units
        
        Arguments:
        * from_units: default 'micros'. Must be in the list of list_of_time_units
        * to_units: default 's'. Must be in the list of list_of_time_units
        
        Returns:
        * coefficient to apply to data to convert from first units to second units provided
        
        Raises:
        * ValueError: if any of the units is not supported
        """

        if (not (from_units in Utilities.list_of_time_units) or
        not(to_units in Utilities.list_of_time_units)):
            raise ValueError("Units not supported")

        if from_units == to_units:
            return 1
        
        if from_units == 's':
            if to_units == 'ms': return 1.e3
            if to_units == 'micros': return 1.e6
            if to_units == 'ns': return 1.e9
            
        if from_units == 'ms':
            if to_units == 's': return 1.e-3
            if to_units == 'micros': return 1.e3
            if to_units == 'ns': return 1.e6

        if from_units == 'micros':
            if to_units == 's': return 1.e-6
            if to_units == 'ms': return 1.e-3
            if to_units == 'ns': return 1.e3

        if from_units == 'ns':
            if to_units == 's': return 1.e-9
            if to_units == 'ms': return 1.e-6
            if to_units == 'micros': return 1.e-3

    @staticmethod
    def array_multiply_coeff(data=None, coeff=1):
        """multiply each element of the array by the coeff
        
        Parameters:
        * data: array to apply coefficient on
        * coeff: default value is 1. Coefficient to apply
        
        Returns:
        * data * coefficient
        """
        if data is None:
            raise ValueError("Give me at least something to multiply!")
    
        final_data = np.array([])
        for _item in data:
            _value = float(_item) * float(coeff)
            final_data = np.append(final_data, _value)
            
        return final_data
    
    @staticmethod
    def array_add_coeff(data=None, coeff=1.):
        """Add coefficient to each element of the array
        
        Parameters:
        * data: array to apply coefficient on
        * coeff: default value is 1. Coefficient to apply
        
        Returns:
        * data + coefficient
        """
        if data is None:
            raise ValueError("Give ma at least something to add!")
        
        final_data = np.array([])
        for index in range(len(data)):
            _item = data[index]
            _value = _item + coeff
            final_data = np.append(final_data, _value)
            
        return final_data
           
    @staticmethod
    def array_divide_array(numerator=None, denominator=None):
        """Divide two arrays of the same size
        
        Parameters:
        * numerator: numpy array
        * denominator: numpy array
        
        Returns:
        * numerator / denominator
        
        Raises:
        * ValueError if array do not have the same size
        """
        if len(numerator) != len(denominator):
            raise ValueError("Arrays do not have the same size!")
        
        return numerator / denominator
            
    @staticmethod
    def array_minus_array(array1=None, array2=None):
        """Substract second array from first array provided
        
        Parameters:
        * array1: left side of the '-' operator
        * array2: right side of the '-' operator
        
        Returns:
        * Array1 - Array2
        
        Raises:
        * ValueError if arrays do not have the same size
        """
        if len(array1) != len(array2):
            raise ValueError("Arrays do not have the same size!")
        
        return array1 - array2
            
    @staticmethod
    def load_csv(filename=None):
        """Load a csv file and return its content
        
        Parameters:
        * filename: name of the csv file to load
        
        Returns:
        contents of the file as an array item for each line
        
        Raise:
        ValueError if format is wrong
        
        """
        _input_file = filename
        try:
            f = open(_input_file, 'r')
            _tof = []
            for _line in f:
                if '#' in _line:
                    continue
                _value = float(_line.strip())
                _tof.append(_value)
            return _tof
        except:
            raise ValueError("Bad file format")        
        
    @staticmethod
    def load_ascii(filename=None, sep=''):
        """Load an ascii file using the separator provided to separete the value in the same row
        
        Parameters:
        * filename: ascii full file name of file to load
        * sep: default ' '. Separator to use to separate values in the same row
        
        Returns:
        Array of values 
        
        Raise:
        ValueError if file does not exist or format is wrong
        """
        _input_file = filename
        try:
            _final_array = np.genfromtxt(_input_file, 
                                         delimiter = sep)
            return _final_array
        except:
            raise ValueError("Bad file format!")
        
    @staticmethod
    def save_csv(filename=None, metadata=None, data=None):
        """Create comma separated file (CSV)
        
        Arguments:
        * filename: name of output file
        * metadata: metadata string array (will be placed at the top of the file with '#' in front)
        * data: data float array"""
        sep = ", "
        f = open(filename, "w")
        for _meta in metadata:
            f.write("# " + _meta +  "\n")
        for _row in data:
            if (type(_row) is np.ndarray) or (type(_row) is list):
                _row_string = [str(x) for x in _row]
                _row_format = sep.join(_row_string)
            else:
                _row_format = str(_row)
            f.write(_row_format + "\n")
        f.close()        


import unittest
import os
import numpy as np
from neutronbraggedge.experiment_handler.tof import TOF


class TofTest(unittest.TestCase):

    def setUp(self):
        _file_path = os.path.dirname(__file__)
        self.data_path = os.path.abspath(os.path.join(_file_path, '../../data'))
    
    def get_full_path(self, file_name):
        return os.path.join(self.data_path, file_name)

    def test_loading_manual_tof_in_s_units(self):
        """Assert in TOF - TOF(s) array is correctly manually loaded"""
        _tof_array = [1., 2., 3., 4., 5., 6., 7., 8., 9.]
        _tof_handler = TOF(tof_array = _tof_array)
        self.assertTrue(all(_tof_array == _tof_handler.tof_array))

    def test_loading_manual_tof_raise_error_if_no_data_provided(self):
        """Assert in TOF - that ValueError is raised if not tof array provided"""
        self.assertRaises(ValueError, TOF)
        
    def test_loading_file_raise_error_if_file_does_not_exist(self):
        """Assert in TOF - that IOError is raised when file does not exist"""
        _filename = self.get_full_path('fake_tof.txt')
        self.assertRaises(IOError, TOF, _filename)

    def test_loading_manual_tof_in_micros_units(self):
        """Assert in TOF - TOF(micros) array is correctly manually loaded and units are converted"""
        _tof_array = np.array([1.e6, 2.e6, 3.e6, 4.e6, 5.e6, 6.e6, 7.e6, 8.e6, 9.e6])
        _tof_units = 'micros'
        _tof_handler = TOF(tof_array = _tof_array, units = _tof_units)
        self.assertTrue(all(_tof_array*1.e-6 == _tof_handler.tof_array))
        
    def test_loading_manual_tof_in_ms_units(self):
        """Assert in TOF - TOF(ms) array is correctly manually loaded and units are converted"""
        _tof_array = np.array([1.e3, 2.e3, 3.e3, 4.e3, 5.e3, 6.e3, 7.e3, 8.e3, 9.e3])
        _tof_units = 'ms'
        _tof_handler = TOF(tof_array = _tof_array, units = _tof_units)
        self.assertTrue(all(_tof_array*1.e-3 == _tof_handler.tof_array))

    def test_loading_manual_tof_in_ns_units(self):
        """Assert in TOF - TOF(ms) array is correctly manually loaded and units are converted"""
        _tof_array = np.array([1.e9, 2.e9, 3.e9, 4.e9, 5.e9, 6.e9, 7.e9, 8.e9, 9.e9])
        _tof_units = 'ns'
        _tof_handler = TOF(tof_array = _tof_array, units = _tof_units)
        self.assertTrue(all(_tof_array*1.e-9 == _tof_handler.tof_array))

    def test_loading_manual_tof_units_not_implemented_yet(self):
        """Assert in TOF - that an error is thrown when the units is not recognized"""
        _tof_array = np.array([1.e9, 2.e9, 3.e9, 4.e9, 5.e9, 6.e9, 7.e9, 8.e9, 9.e9])
        _tof_units = 'crazys'
        self.assertRaises(NotImplementedError, None, TOF, _tof_array, _tof_units)

    def test_loading_good_tof_file(self):
        """Assert in TOF - that correctly formated tof file is correctly loaded"""
        _filename = self.get_full_path('good_tof.txt')
        _tof_handler = TOF(filename = _filename)
        _tof_expected = np.array([1.0, 2.0, 3.0, 4.0])
        self.assertTrue(all(_tof_expected == _tof_handler.tof_array[0:4]))

    def test_loading_real_tof_file(self):
        """Assert in TOF - that real tof file is correctly loaded"""
        _filename = self.get_full_path('tof.txt')
        _tof_handler = TOF(filename = _filename)
        _tof_expected = np.array([9.6e-7, 1.12e-5, 2.144e-5, 3.168e-5])
        self.assertTrue(all(_tof_expected == _tof_handler.tof_array[0:4]))
        
    def test_loading_counts_column(self):
        """Assert in TOF - second column (counts) is correctly loaded"""
        _filename = self.get_full_path('tof.txt')
        _tof_handler = TOF(filename = _filename)
        _counts_expected = np.array([2137, 1988, 1979, 2078])
        self.assertTrue(all(_counts_expected == _tof_handler.counts_array[0:4]))


if __name__ == '__main__':
    unittest.main()
